// mcx_fix8_test.cpp : main project file.

#include "stdafx.h"

#include <fix8/f8includes.hpp>
#include <fix8/usage.hpp>
#include "mcx44_types.hpp"
#include "mcx44_router.hpp"
#include "mcx44_classes.hpp"

#include "mcx_fix8_test.h"


#include <msclr/marshal_cppstd.h>

using namespace System;



int main(array<System::String ^> ^args)
{	
	System::IO::FileInfo ^fi = gcnew System::IO::FileInfo(System::Reflection::Assembly::GetEntryAssembly()->Location);
	String^ s = fi->DirectoryName + "\\mcx_cur_fx8.xml";

	std::string cfg_file = msclr::interop::marshal_as< std::string >( s);

    Console::WriteLine(L"Starting FIX session");
		
	FIX8::ClientSession<mcx_session_client>* mSession(
		new FIX8::ClientSession<mcx_session_client>(FIX8::FIX44::ctx(), cfg_file, "MCXFIX", false));

	mSession->start(false, 0U, 0U, mSession->session_ptr()->get_login_parameters()._davi());

	Console::WriteLine(L"Started successfully");
	Console::WriteLine("----------------------------------------------");
	Console::WriteLine(L"Press 's' to send sample message or 'q' to quit");
	Console::WriteLine("----------------------------------------------");

	Char c = 0;

	do 
	{
		c = Console::ReadKey().KeyChar;

		if (c == 'q')
			break;

		if (c == 's')
		{
			Console::Write("\r");

			F8SessionWrapper ^wrp = gcnew F8SessionWrapper(mSession);
			wrp->SendMessage();
		}
	}
	while (true);

    return 0;
}


bool mcx_router_client::operator() (const FIX8::FIX44::ExecutionReport *msg) const
{
	std::cout << "---------------Execution-Report---------------" << std::endl;
	msg->print(std::cout);
	std::cout << "----------------------------------------------" << std::endl;
	return true;
}

FIX8::Message* mcx_session_client::generate_logon(const unsigned heartbeat_interval, const FIX8::f8String davi)
{
	FIX8::Message* msg = FIX8::Session::generate_logon(heartbeat_interval, davi);
	if (msg) *msg << new FIX8::FIX44::Password(" ");
	return msg;
}

bool mcx_session_client::handle_logon(const unsigned seqnum, const FIX8::Message *msg) 
{
	std::cout << "--------------------Logon---------------------" << std::endl;
	msg->print(std::cout);
	std::cout << "----------------------------------------------" << std::endl;
	return true;
}

bool mcx_session_client::handle_logout(const unsigned seqnum, const FIX8::Message *msg) 
{
	std::cout << "--------------------Logout--------------------" << std::endl;
	msg->print(std::cout);
	std::cout << "----------------------------------------------" << std::endl;
	return true;
}

bool mcx_session_client::handle_application(const unsigned seqnum, const FIX8::Message *&msg) 
{ 
	/*

	pay attentiont what 'enforce(seqnum, msg)' is commented!!!
	if uncomment - will not work properly

	*/

	return /*enforce(seqnum, msg) ||*/ msg->process(_router); 
}

void F8SessionWrapper::SendMessage()
{
	System::Threading::ThreadPool::QueueUserWorkItem(
		gcnew System::Threading::WaitCallback(this, &F8SessionWrapper::OnSendMessage));
}

void F8SessionWrapper::OnSendMessage(Object ^state)
{
	try
	{
		Console::WriteLine("Sending NewOrderSingle");

		FIX8::FIX44::NewOrderSingle *message(new FIX8::FIX44::NewOrderSingle());

		*message 
			<< new FIX8::FIX44::ClOrdID("1111111111")
			<< new FIX8::FIX44::SecondaryClOrdID("1234567");

		*message << new FIX8::FIX44::NoPartyIDs(1);
		FIX8::GroupBase *nopartyids(message->find_group<FIX8::FIX44::NewOrderSingle::NoPartyIDs>());
		FIX8::MessageBase *gr1(nopartyids->create_group());
		*gr1 << new FIX8::FIX44::PartyID("QINVESTOR")
				<< new FIX8::FIX44::PartyIDSource('D')
				<< new FIX8::FIX44::PartyRole(3);
		*nopartyids << gr1;
	

		*message 
			<< new FIX8::FIX44::Account("MB01382CURR0")
			<< new FIX8::FIX44::NoTradingSessions(1);

		FIX8::GroupBase *notrsessions(message->find_group<FIX8::FIX44::NewOrderSingle::NoTradingSessions>());
		FIX8::MessageBase *gr2(notrsessions->create_group());
		*gr2 << new FIX8::FIX44::TradingSessionID("CETS");
		*notrsessions << gr2;
			
		*message
			<< new FIX8::FIX44::Symbol("USD000UTSTOM")
			<< new FIX8::FIX44::Side(FIX8::FIX44::Side_BUY)
			<< new FIX8::FIX44::TransactTime()
			<< new FIX8::FIX44::OrderQty(1)
			<< new FIX8::FIX44::OrdType(FIX8::FIX44::OrdType_LIMIT)
			<< new FIX8::FIX44::Price(35.55)
			<< new FIX8::FIX44::TimeInForce(FIX8::FIX44::TimeInForce_FILLORKILL);

		_session->session_ptr()->send(*message, true);
		
		Console::WriteLine("Successfully sent");
	}
	catch (Exception^ ex)
	{
		Console::WriteLine(ex->ToString());
	}
	finally
	{
		if (_handle.IsAllocated) _handle.Free();
	}
}